@@ -3,7 +3,7 @@ package CGI::Compile;
 use strict;
 use 5.008_001;
 
-our $VERSION = '0.21';
+our $VERSION = '0.23';
 
 use Cwd;
 use File::Basename;
@@ -167,12 +167,15 @@ sub _build_package {
     return $package;
 }
 
-# we use a tmpdir chmodded to 0700 so that the tempfiles are secure
-my $tmp_dir = File::Spec->catfile(File::Spec->tmpdir, "cgi_compile_$$");
-
+# define tmp_dir value later on first usage, otherwise all children
+# share the same directory when forked
+my $tmp_dir;
 sub _eval {
     my $code = \$_[1];
 
+    # we use a tmpdir chmodded to 0700 so that the tempfiles are secure
+    $tmp_dir ||= File::Spec->catfile(File::Spec->tmpdir, "cgi_compile_$$");
+
     if (! -d $tmp_dir) {
         mkdir $tmp_dir          or die "Could not mkdir $tmp_dir: $!";
         chmod 0700, $tmp_dir    or die "Could not chmod 0700 $tmp_dir: $!";
@@ -192,7 +195,7 @@ sub _eval {
 }
 
 END {
-    if (-d $tmp_dir) {
+    if ($tmp_dir and -d $tmp_dir) {
         File::Path::remove_tree($tmp_dir);
     }
 }
@@ -515,7 +518,7 @@ Torsten Förtsch E<lt>torsten.foertsch@gmx.netE<gt>
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright (c) 2009 Tatsuhiko Miyagawa E<lt>miyagawa@bulknews.netE<gt>
+Copyright (c) 2020 Tatsuhiko Miyagawa E<lt>miyagawa@bulknews.netE<gt>
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
